
local util = {}

local lfs = require("lfs")
local stat = require("posix.sys.stat")

function util.write_file(name, data)
   local fd = io.open(name, "w")
   if not fd then
      return nil, "could not write "..name
   end
   fd:write(data)
   fd:close()
   return true
end

function util.read_file(name, offset)
   offset = offset or 0
   local fd, err = io.open(name, "r")
   if not fd then
      return nil, "could not read "..name.." : "..err
   end
   local n_bytes, err = fd:seek("set", offset)
   if not n_bytes then
      return nil, "could not read "..name.." : "..err
   end
   local data = fd:read("*a")
   fd:close()
   return data
end

function util.debug_table(tbl, lvl)
   lvl = lvl or 0
   local indent = ("   "):rep(lvl)
   local out = {"{\n"}
   for k,v in pairs(tbl) do
      table.insert(out, indent.."   ")
      table.insert(out, tostring(k).." = ")
      table.insert(out, (type(v) == "table" and util.debug_table(v, lvl + 1) or tostring(v)))
      table.insert(out, "\n")
   end
   table.insert(out, indent.."}")
   return table.concat(out)
end

function util.collect_exec_data(job)
    local script = job.parameters.csbase_command_path.."/collect_execution_data"
    if not stat.stat(script) then
        return pairs({})
    end

    return coroutine.wrap(function ()
        local params = {
            job.cmd_id,
            job.sandboxes[1],
            job.parameters.csbase_command_path,
            job.parameters.csbase_command_output_path,
            job.parameters.csbase_command_root_path
        }
        local collect_cmd = "ksh "..script.." "..table.concat(params," ")
        --logger.debug("Executing data collection script: "..collect_cmd)
        local stdout = assert(io.popen(collect_cmd, 'r'))

        for line in stdout:lines() do
            for key, value in string.gmatch(line, "(.-)%s*=%s*([^,]+)") do
                coroutine.yield(key, value)
            end
        end
        stdout:close()
    end)
end

local function can_open(path, mode)
   local fd = io.open(path, mode)
   return fd and fd:close() or false
end

function util.list_local_path(path_name)
   local attrs = lfs.attributes(path_name)
   local list = {}
   if attrs and attrs.mode == "directory" then
      for file_name in lfs.dir(path_name) do
         if file_name ~= "." and file_name ~= ".." then
            local file_path
            if path_name:sub(-1) == "/" then
               file_path = path_name..file_name
            else
               file_path = path_name.."/"..file_name
            end
            list[#list + 1] = util.check_local_path(file_path)
         end
      end
      return list
   end	   
   return nil, "Could not list path"
end

function util.check_local_path(path_name)
   local symlink_attrs = lfs.symlinkattributes(path_name)
   if symlink_attrs then
      local size = symlink_attrs.size
      local is_dir = symlink_attrs.mode == "directory"
      -- in case of links, get data from the target file
      if symlink_attrs.mode == "link" then
         local attrs = lfs.attributes(path_name)
         if attrs then
            --print(util.debug_table(attrs))
            size = attrs.size
            is_dir = attrs.mode == "directory"
         end
      end
      -- when trying to open a pipe/socket for permission check, the daemon can get stuck
      local is_readable = symlink_attrs.mode ~= "named pipe" and symlink_attrs.mode ~= "socket" and can_open(path_name, "r")
      local is_writable = symlink_attrs.mode ~= "named pipe" and symlink_attrs.mode ~= "socket" and can_open(path_name, "a")
      --print(util.debug_table(symlink_attrs))
      return {
         path = path_name,
         size_bytes = size,
         is_dir = is_dir,
         is_symlink = symlink_attrs.mode == "link",
         is_readable = is_readable,
         is_writable = is_writable,
         is_executable = true, --FIXME not obtainable using lfs
         link_path = symlink_attrs.target or path_name,
      }
   end
   return nil, "Could not read path"
end

function util.convert_to_list(map, key)
   local i = 1
   local tab = {}
   while (map[key.."."..i]) do
      table.insert(tab, map[key.."."..i])
      i = i + 1
  end
  return tab
end

function util.removedir(path)
    for file in lfs.dir(path) do
        if file ~= "." and file ~= ".." then
            local f = path..'/'..file
            local attr = lfs.attributes(f)
            assert(attr)
            if attr.mode == "directory" then
                util.removedir(f)
            else
                local ok, err = os.remove(f)
                if not ok then
                    return ok, err
                end
            end
        end
    end
    return lfs.rmdir(path)
end

return util
